@@ -1,4 +1,4 @@
-/* arp.h - ARP state machine */
+/* arp.c - ARP state machine */
  
 /* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
@@ -11,6 +11,7 @@
 #include <sys/types.h>
 #include <sys/socket.h> /* for linux/if_arp.h */
 #include <netinet/in.h> /* for ntohs, etc. */
+#include <linux/netdevice.h> /* for MAX_ADDR_LEN for if_arp.h */
 #include <linux/if_arp.h>
 #include <linux/if_ether.h>
 #include <linux/atm.h>
@@ -62,7 +63,7 @@ static void discard_vcc(VCC *vcc)
 }
 
 
-static void discard_entry(ENTRY *entry)
+void discard_entry(ENTRY *entry)
 {
     VCC *vcc,*next;
 
@@ -99,7 +100,7 @@ static int want_arp_srv(ITF *itf)
 	if (!vcc->connecting) return 1;
     if (itf->arp_srv->vccs) return 0;
     if ((fd = connect_vcc((struct sockaddr *) itf->arp_srv->addr,
-      CLIP_DEFAULT_IDLETIMER)) < 0) return 0;
+      &itf->arp_srv->qos,CLIP_DEFAULT_IDLETIMER)) < 0) return 0;
     vcc = alloc_t(VCC);
     vcc->active = 1;
     vcc->connecting = 1;
@@ -295,8 +296,18 @@ static void timer(ENTRY *entry)
 	    arp_request(entry->itf,entry->ip);
 	    break;
 	case as_invalid:
-	    inarp_request(entry);
+	    if (entry->vccs) inarp_request(entry);
+	    else if (!(entry->flags & ATF_ARPSRV))
+		    diag(COMPONENT,DIAG_ERROR,"as_invalid, no VCCs, but not "
+		      "ARP server entry");
+		    else if (!want_arp_srv(entry->itf)) STOP_TIMER(entry);
 	    break;
+	case as_valid:
+	    if (entry->flags & ATF_ARPSRV) {
+		if (!want_arp_srv(entry->itf)) STOP_TIMER(entry);
+		break;
+	    }
+	    /* fall through */
 	default:
 	    diag(COMPONENT,DIAG_FATAL,"timer in state %d",entry->state);
     }
@@ -329,7 +340,8 @@ static void timeout(ENTRY *entry)
 		    if (set_ip(vcc->fd,0) < 0)
 			diag(COMPONENT,DIAG_ERROR,"set_ip(0): %s",
 			  strerror(errno));
-	    if (entry->svc && entry->itf->arp_srv) revalidate(entry);
+	    if (entry->svc && entry->itf->arp_srv &&
+	      !(entry->flags & ATF_ARPSRV)) revalidate(entry);
 	    else {
 		inarp_request(entry);
 		START_TIMER(entry,REPLY);
@@ -379,7 +391,7 @@ static void connect_me(ENTRY *entry)
     VCC *vcc;
     int fd;
 
-    if ((fd = connect_vcc((struct sockaddr *) entry->addr,
+    if ((fd = connect_vcc((struct sockaddr *) entry->addr,&entry->qos,
       CLIP_DEFAULT_IDLETIMER)) < 0) return;
     vcc = alloc_t(VCC);
     vcc->active = 1;
@@ -429,14 +441,7 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
     }
     else {
-	entry = alloc_t(ENTRY);
-	entry->svc = 1;
-	entry->state = as_none;
-	entry->addr = NULL;
-	entry->flags = 0;
-	entry->timer = NULL;
-	entry->vccs = NULL;
-	entry->itf = NULL;
+	entry = alloc_entry(1);
 	if (vcc) {
 	    vcc->entry = entry;
 	    Q_INSERT_HEAD(entry->vccs,vcc);
@@ -454,10 +459,10 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
     if (!entry->addr) entry->addr = alloc(sizeof(*addr));
     *entry->addr = *addr;
     /* need to revise this policy if we get user-settable QOS @@@ */
-    memset(&entry->addr->sas_txtp,0,sizeof(entry->addr->sas_txtp));
-    entry->addr->sas_txtp.class = ATM_UBR;
-    entry->addr->sas_txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
-    entry->addr->sas_rxtp = entry->addr->sas_txtp;
+    memset(&entry->qos,0,sizeof(entry->qos));
+    entry->qos.txtp.class = ATM_UBR;
+    entry->qos.txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
+    entry->qos.rxtp = entry->qos.txtp;
     for (walk = entry->vccs; walk; walk = walk->next)
 	if (!walk->connecting)
 	    if (set_ip(walk->fd,ip) < 0)
@@ -508,18 +513,15 @@ void need_ip(int itf_num,unsigned long ip)
 		if (!entry->vccs) connect_me(entry);
 		return;
 	    case as_invalid:
-		if (entry->svc && entry->itf && entry->itf->arp_srv) break;
+		if (entry->svc && entry->itf && entry->itf->arp_srv &&
+		    !(entry->flags & ATF_ARPSRV)) break;
 		return;
 	    default:
 		diag(COMPONENT,DIAG_FATAL,"bad state %d",entry->state);
 	}
     else {
-	entry = alloc_t(ENTRY);
-	entry->svc = 1;
+	entry = alloc_entry(1);
 	entry->ip = ip;
-	entry->addr = NULL;
-	entry->flags = 0;
-	entry->vccs = NULL;
 	entry->itf = itf;
 	Q_INSERT_HEAD(itf->table,entry);
     }
@@ -668,27 +670,22 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
 
 
 static int ioctl_set_pvc(ITF *itf,unsigned long ip,
-  struct sockaddr_atmpvc *addr,int flags)
+  struct sockaddr_atmpvc *addr,const struct atm_qos *qos,int flags)
 {
     ENTRY *entry;
     VCC *vcc;
     int fd,result;
 
     if (lookup_ip(itf,ip)) return -EEXIST; 
-    if ((fd = connect_vcc((struct sockaddr *) addr,0)) < 0) return fd;
+    if ((fd = connect_vcc((struct sockaddr *) addr,qos,0)) < 0) return fd;
     if ((result = set_ip(fd,ip)) < 0) return result;
     if (flags & ATF_NULL)
 	if ((result = set_encap(fd,0)) < 0) return result;
-    entry = alloc_t(ENTRY);
+    entry = alloc_entry(0);
     entry->state = as_valid;
-    entry->svc = 0;
     entry->ip = ip;
-    entry->addr = NULL;
     entry->flags = flags;
-    entry->timer = NULL;
-    entry->vccs = NULL;
     entry->itf = itf;
-    entry->vccs = NULL;
     vcc = alloc_t(VCC);
     vcc->active = 1;
     vcc->connecting = 0;
@@ -702,23 +699,20 @@ static int ioctl_set_pvc(ITF *itf,unsigned long ip,
 
 
 static int ioctl_set_svc(ITF *itf,unsigned long ip,
-  struct sockaddr_atmsvc *addr,int flags)
+  struct sockaddr_atmsvc *addr,const struct atm_qos *qos,int flags)
 {
     ENTRY *entry;
 
     if (flags & ATF_ARPSRV) flags |= ATF_PERM;
     if (lookup_ip(itf,ip)) return -EEXIST;
-    entry = alloc_t(ENTRY);
+    entry = alloc_entry(1);
     entry->state = as_valid;
-    entry->svc = 1;
     entry->ip = ip;
     entry->addr = alloc_t(struct sockaddr_atmsvc);
     *entry->addr = *addr;
-diag(COMPONENT,DIAG_DEBUG,"tx-sdu: %d",addr->sas_txtp.max_sdu);
+    entry->qos = *qos;
     entry->flags = flags;
-    entry->timer = NULL;
     if (!(flags & ATF_PERM) || (flags & ATF_ARPSRV)) START_TIMER(entry,REVAL);
-    entry->vccs = NULL;
     entry->itf = itf;
     Q_INSERT_HEAD(itf->table,entry);
     if (flags & ATF_ARPSRV) {
@@ -730,7 +724,7 @@ diag(COMPONENT,DIAG_DEBUG,"tx-sdu: %d",addr->sas_txtp.max_sdu);
 }
 
 
-static int ioctl_delete(ITF *itf,unsigned long ip)
+static int ioctl_delete(ITF *itf,unsigned long ip,int flags)
 {
     ENTRY *entry,*walk,*next;
 
@@ -738,6 +732,7 @@ static int ioctl_delete(ITF *itf,unsigned long ip)
 	diag(COMPONENT,DIAG_WARN,"ioctl_delete didn't find entry");
 	return -ENOENT;
     }
+    if ((flags ^ entry->flags) & ATF_ARPSRV) return -EINVAL;
     if ((entry->flags & ATF_ARPSRV) && entry->itf)
 	for (walk = entry->itf->table; walk; walk = next) {
 	    next = walk->next;
@@ -790,11 +785,11 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
 		case AF_ATMPVC:	
 		    return ioctl_set_pvc(itf,((struct sockaddr_in *)
 		      &req->arp_pa)->sin_addr.s_addr,(struct sockaddr_atmpvc *)
-		      &req->arp_ha,req->arp_flags);
+		      &req->arp_ha,&req->arp_qos,req->arp_flags);
 		case AF_ATMSVC:
 		    return ioctl_set_svc(itf,((struct sockaddr_in *)
 		      &req->arp_pa)->sin_addr.s_addr,(struct sockaddr_atmsvc *)
-		      &req->arp_ha,req->arp_flags);
+		      &req->arp_ha,&req->arp_qos,req->arp_flags);
 		default: /* not reached - atm2text complains before */
 		    diag(COMPONENT,DIAG_ERROR,"SIOCSARP: bad HA AF 0x%x",
 		      req->arp_ha.sas_family);
@@ -806,7 +801,7 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
 	case SIOCDARP:
 	    diag(COMPONENT,DIAG_DEBUG,"got SIOCDARP for itf %d, IP %d.%d.%d.%d"
 	      ,itf_num,ipp[0],ipp[1],ipp[2],ipp[3]);
-	    return ioctl_delete(itf,ip);
+	    return ioctl_delete(itf,ip,req->arp_flags);
 	default:
 	    diag(COMPONENT,DIAG_ERROR,"unrecognized ioctl 0x%x",cmd);
 	    return -EINVAL;
