@@ -1,6 +1,6 @@
 /* arp.h - ARP state machine */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef ARP_H
@@ -11,6 +11,7 @@
 #include "table.h"
 
 
+void discard_entry(ENTRY *entry);
 void need_ip(int itf_num,unsigned long ip);
 void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len);
 int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req);
