@@ -218,18 +218,23 @@ int send_message(int fd, Message *message)
 int open_ilmi(int itf)
 {
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     int fd;
  
     if((fd = socket(PF_ATMPVC, SOCK_DGRAM, ATM_AAL5)) < 0)
       diag(COMPONENT, DIAG_FATAL, "socket: %s", strerror(errno));
 
+    memset(&qos, 0, sizeof(qos));
+    qos.rxtp.class = ATM_UBR;
+    qos.txtp.class = ATM_UBR;
+    qos.rxtp.max_sdu = MAX_ILMI_MSG;
+    qos.txtp.max_sdu = MAX_ILMI_MSG;
+    if (setsockopt(fd, SOL_ATM, SO_ATMQOS, &qos, sizeof(qos)) < 0)
+	diag(COMPONENT,DIAG_FATAL,"setsockopt SO_ATMQOS: %s",strerror(errno));
+
     memset(&addr, 0, sizeof(addr));
     addr.sap_family = AF_ATMPVC;
     addr.sap_addr.itf = itf;
-    addr.sap_rxtp.class = ATM_UBR;
-    addr.sap_txtp.class = ATM_UBR;
-    addr.sap_rxtp.max_sdu = MAX_ILMI_MSG;
-    addr.sap_txtp.max_sdu = MAX_ILMI_MSG;
     addr.sap_addr.vpi = 0;
     addr.sap_addr.vci = SNMP_VCI;
 
