@@ -19,6 +19,7 @@
 #include <linux/atm.h>
 #include <linux/atmarp.h>
 #include <linux/atmclip.h>
+#include <linux/netdevice.h> /* for MAX_ADDR_LEN for if_arp.h */
 #include <linux/if_arp.h> /* for ATF_PERM */
 
 #include "atm.h"
@@ -57,7 +58,7 @@ static void usage(const char *name)
     fprintf(stderr,"%6s %s -c [atmN]\n","",name);
     fprintf(stderr,"%6s %s -s ip_addr [itf.]vpi.vci [temp] [null]\n","",name);
     fprintf(stderr,"%6s %s -s ip_addr atm_addr [temp] [arpsrv]\n","",name);
-    fprintf(stderr,"%6s %s -d ip_addr\n","",name);
+    fprintf(stderr,"%6s %s -d ip_addr [arpsrv]\n","",name);
     exit(1);
 }
 
@@ -118,7 +119,7 @@ int main(int argc,char **argv)
 	    if (argc < optind+2) usage(argv[0]);
 	    break;
 	case SIOCDARP:
-	    if (argc != optind+1) usage(argv[0]);
+	    if (argc < optind+1) usage(argv[0]);
 	    break;
 	default:
 	    usage(argv[0]);
@@ -143,8 +144,8 @@ int main(int argc,char **argv)
 	memcpy(&((struct sockaddr_in *) &req.arp_pa)->sin_addr,hostent->h_addr,
 	  hostent->h_length);
     }
+    req.arp_flags = 0;
     if (op == SIOCSARP) {
-	req.arp_flags = 0;
 	for (i = optind+2; i < argc; i++)
 		if (!strcmp(argv[i],"term")) req.arp_flags |= ATF_PERM;
 		else if (!strcmp(argv[i],"null")) req.arp_flags |= ATF_NULL;
@@ -153,22 +154,17 @@ int main(int argc,char **argv)
 	req.arp_flags ^= ATF_PERM;
 	if (text2atm(argv[optind+1],(struct sockaddr *) &req.arp_ha,
 	  sizeof(req.arp_ha),T2A_NAME) < 0) {
-	    fprintf(stderr,"%s: invalid ATM address\n",argv[i]);
+	    fprintf(stderr,"%s: invalid ATM address\n",argv[optind+1]);
 	    return 1;
 	}
-	if (((struct sockaddr *) &req.arp_ha)->sa_family == AF_ATMPVC) {
-	    ((struct sockaddr_atmpvc *) &req.arp_ha)->sap_txtp.class = ATM_UBR;
-	    ((struct sockaddr_atmpvc *) &req.arp_ha)->sap_txtp.max_sdu =
-	      RFC1483LLC_LEN+RFC1626_MTU;
-	    ((struct sockaddr_atmpvc *) &req.arp_ha)->sap_rxtp =
-	      ((struct sockaddr_atmpvc *) &req.arp_ha)->sap_txtp;
-	}
-	else {
-	    req.arp_ha.sas_txtp.class = ATM_UBR;
-	    req.arp_ha.sas_txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
-	    req.arp_ha.sas_rxtp = req.arp_ha.sas_txtp;
-	/* @@@ special treatment for SVCs ... */
-	}
+	memset(&req.arp_qos,0,sizeof(req.arp_qos));
+	req.arp_qos.txtp.class = ATM_UBR;
+	req.arp_qos.txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
+	req.arp_qos.rxtp = req.arp_qos.txtp;
+    }
+    if (op == SIOCDARP && optind+1 < argc) {
+	if (optind+2 < argc || strcmp(argv[optind+1],"arpsrv")) usage(argv[0]);
+	req.arp_flags |= ATF_ARPSRV;
     }
     if ((s = socket(AF_INET,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
