@@ -1,6 +1,6 @@
 /* clip.c - CLassical IP over ATM control (to be replaced by ATMARP) */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -26,6 +26,7 @@ static void usage(const char *name)
 int main(int argc,char **argv)
 {
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     const char *name;
     int s,null;
 
@@ -50,14 +51,19 @@ int main(int argc,char **argv)
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
-    if (argc != 3) addr.sap_txtp.class = ATM_UBR;
+    memset(&qos,0,sizeof(qos));
+    if (argc != 3) qos.txtp.class = ATM_UBR;
     else {
-	addr.sap_txtp.class = ATM_CBR;
-	addr.sap_txtp.min_pcr = atoi(argv[2]);
-	addr.sap_txtp.max_sdu = RFC1626_MTU;
+	qos.txtp.class = ATM_CBR;
+	qos.txtp.min_pcr = atoi(argv[2]);
+	qos.txtp.max_sdu = RFC1626_MTU;
+    }
+    qos.rxtp.class = ATM_UBR;
+    qos.rxtp.max_sdu = RFC1626_MTU;
+    if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
+	perror("setsockopt SO_ATMQOS");
+	return 1;
     }
-    addr.sap_rxtp.class = ATM_UBR;
-    addr.sap_rxtp.max_sdu = RFC1626_MTU;
     if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return 1;
