@@ -26,7 +26,7 @@
  *
  * Client registration/query funcs used in Digital's code
  *
- * $Id: address.c,v 1.11 1996/04/25 19:42:13 carnil Exp carnil $
+ * $Id: address.c,v 1.12 1996/07/07 11:51:47 carnil Exp carnil $
  *
  */
 
@@ -58,7 +58,10 @@
 
 #define EMOD MOD_ADDR_REG
 #define EINST "address.c"
-#define EMASK (EM_SERR | EM_NERR)
+
+#include "emask.h"
+
+#define ADDR_TIMER_FREQ 5000
 
 /* One-way list. There won't be that many clients registered.. */
 typedef struct _client_t_ {
@@ -67,12 +70,15 @@ typedef struct _client_t_ {
   char *name; /* Describing text string */
   ADDR_ATM addr; /* ATM address for this client */  
   struct _client_t_ *next; 
+  HANDLE my_timer; /* See that ATM address is maintained */
 } Client_t;
 
 Client_t *Clientlist = NULL;
 
 extern int errno;
 
+void addr_reg_timer_callback(HANDLE context);
+
 STATUS 
 addr_reg_client_register (HANDLE addr_reg_handle, ADDR_REG_CALLBACK callback,
 			  HANDLE callback_handle, const char *p_text,
@@ -110,61 +116,20 @@ addr_reg_client_register (HANDLE addr_reg_handle, ADDR_REG_CALLBACK callback,
   memset(&to_register->addr,0,sizeof(ADDR_ATM));
   to_register->next = Clientlist;
   Clientlist = to_register;
-  return STATUS_K_SUCCESS;
+  return os_timer_alloc(addr_reg_timer_callback, to_register, 
+			&to_register->my_timer);
 }
 
 STATUS 
 addr_reg_atm_addr_alloc(HANDLE client_handle)
 {
   Client_t *client;
-  struct sockaddr_atmsvc ouraddr;
-  struct atmif_sioc req;
-  int s;
 
   EVENT(EM_DEBUG,("Addr_reg_atm_addr_alloc\n"));
 
   client = (Client_t *)client_handle;
-  /* Call callback func when this happens */
-  
-  req.number = 0;
-  req.arg = &ouraddr;
-  req.length = sizeof(ouraddr);
 
-  s = socket(PF_ATMSVC, SOCK_DGRAM, 0);
-  if (s<0) {
-    EVENT(EM_NERR,("Socket failed:%s\n",strerror(errno)));
-    return STATUS_K_PENDING;
-  }
-  if (ioctl(s, ATM_GETADDR, &req) <0) {
-    close(s);
-    EVENT(EM_NERR,("Ioctl failed:%s\n",strerror(errno)));
-    return STATUS_K_PENDING;
-  }
-  close(s);
-
-  client->addr.prefix[0] = ouraddr.sas_addr.prv[0];
-  client->addr.prefix[1] = ouraddr.sas_addr.prv[1];
-  client->addr.prefix[2] = ouraddr.sas_addr.prv[2];
-  client->addr.prefix[3] = ouraddr.sas_addr.prv[3];
-  client->addr.prefix[4] = ouraddr.sas_addr.prv[4];
-  client->addr.prefix[5] = ouraddr.sas_addr.prv[5];
-  client->addr.prefix[6] = ouraddr.sas_addr.prv[6];
-  client->addr.prefix[7] = ouraddr.sas_addr.prv[7];
-  client->addr.prefix[8] = ouraddr.sas_addr.prv[8];
-  client->addr.prefix[9] = ouraddr.sas_addr.prv[9];
-  client->addr.prefix[10] = ouraddr.sas_addr.prv[10];
-  client->addr.prefix[11] = ouraddr.sas_addr.prv[11];
-  client->addr.prefix[12] = ouraddr.sas_addr.prv[12];
-  client->addr.esi[0] = ouraddr.sas_addr.prv[13];
-  client->addr.esi[1] = ouraddr.sas_addr.prv[14];
-  client->addr.esi[2] = ouraddr.sas_addr.prv[15];
-  client->addr.esi[3] = ouraddr.sas_addr.prv[16];
-  client->addr.esi[4] = ouraddr.sas_addr.prv[17];
-  client->addr.esi[5] = ouraddr.sas_addr.prv[18];
-  client->addr.sel = ouraddr.sas_addr.prv[19];
-
-  client->callback(client->callback_handle,&client->addr, 
-		   ADDR_REG_EVENT_ATM_ADDR_ALLOCATED);
+  addr_reg_timer_callback((HANDLE)client);
   return STATUS_K_SUCCESS;
 }
 
@@ -183,7 +148,7 @@ addr_reg_atm_addr_dealloc (HANDLE client_handle, ADDR_ATM *p_atm_addr)
   return STATUS_K_SUCCESS;
 }
 
-int 
+int
 address_convert(char* parsestring, ADDR_ATM *atm_addr)
 {
   struct sockaddr_atmsvc tmp;
@@ -215,6 +180,98 @@ address_convert(char* parsestring, ADDR_ATM *atm_addr)
   return 0;
 }
 
+void 
+addr_reg_timer_callback(HANDLE context)
+{
+  Client_t *client = (Client_t *)context;
+  struct sockaddr_atmsvc ouraddr;
+  struct atmif_sioc req;
+  int s;
+
+  EVENT(EM_DEBUG,("Addr_reg_timer_called\n"));
+  req.number = 0;
+  req.arg = &ouraddr;
+  req.length = sizeof(ouraddr);
+
+  /* Reset (or set) timer */
+  os_timer_set(client->my_timer, ADDR_TIMER_FREQ);
+
+  s = socket(PF_ATMSVC, SOCK_DGRAM, 0);
+  if (s<0) {
+    EVENT(EM_NERR,("Socket failed:%s\n",strerror(errno)));
+    return;
+  }
+  if (ioctl(s, ATM_GETADDR, &req) <0) {
+    close(s);
+    EVENT(EM_NERR,("Ioctl failed:%s\n",strerror(errno)));
+    return;
+  }
+  close(s);
+
+  if (req.length ==0 ) { /* No ATM address */
+    if (ATM_EQUAL_NULL(client->addr)) /* Address hasn't been registered 
+				       * and wasn't registered now */
+      return;
+    EVENT(EM_NERR,("Address invalidated\n"));
+    client->callback(client->callback_handle, &client->addr,
+		     ADDR_REG_EVENT_ATM_ADDR_INVALIDATED);
+    ATM_COPY_NULL(client->addr);
+    return;
+  }
+
+  EVENT(EM_DEBUG,("We have address\n"));
+  if (!ATM_EQUAL_NULL(client->addr)){  /* Address has already been registered*/
+    if (client->addr.prefix[0] == ouraddr.sas_addr.prv[0] &&
+	client->addr.prefix[1] == ouraddr.sas_addr.prv[1] &&
+	client->addr.prefix[2] == ouraddr.sas_addr.prv[2] &&
+	client->addr.prefix[3] == ouraddr.sas_addr.prv[3] &&
+	client->addr.prefix[4] == ouraddr.sas_addr.prv[4] &&
+	client->addr.prefix[5] == ouraddr.sas_addr.prv[5] &&
+	client->addr.prefix[6] == ouraddr.sas_addr.prv[6] &&
+	client->addr.prefix[7] == ouraddr.sas_addr.prv[7] &&
+	client->addr.prefix[8] == ouraddr.sas_addr.prv[8] &&
+	client->addr.prefix[9] == ouraddr.sas_addr.prv[9] &&
+	client->addr.prefix[10] == ouraddr.sas_addr.prv[10] &&
+	client->addr.prefix[11] == ouraddr.sas_addr.prv[11] &&
+	client->addr.prefix[12] == ouraddr.sas_addr.prv[12] &&
+	client->addr.esi[0] == ouraddr.sas_addr.prv[13] &&
+	client->addr.esi[1] == ouraddr.sas_addr.prv[14] &&
+	client->addr.esi[2] == ouraddr.sas_addr.prv[15] &&
+	client->addr.esi[3] == ouraddr.sas_addr.prv[16] &&
+	client->addr.esi[4] == ouraddr.sas_addr.prv[17] &&
+	client->addr.esi[5] == ouraddr.sas_addr.prv[18] &&
+	client->addr.sel == ouraddr.sas_addr.prv[19])/*...and has stayed same*/
+      return;
+    EVENT(EM_DEBUG,("... which is new one\n"));
+    /* Hasn't stayed same. First invalidate previous, then register new. */
+    client->callback(client->callback_handle, &client->addr,
+		     ADDR_REG_EVENT_ATM_ADDR_INVALIDATED);
+  }
+  EVENT(EM_DEBUG,("Register new one\n"));
+  client->addr.prefix[0] = ouraddr.sas_addr.prv[0];
+  client->addr.prefix[1] = ouraddr.sas_addr.prv[1];
+  client->addr.prefix[2] = ouraddr.sas_addr.prv[2];
+  client->addr.prefix[3] = ouraddr.sas_addr.prv[3];
+  client->addr.prefix[4] = ouraddr.sas_addr.prv[4];
+  client->addr.prefix[5] = ouraddr.sas_addr.prv[5];
+  client->addr.prefix[6] = ouraddr.sas_addr.prv[6];
+  client->addr.prefix[7] = ouraddr.sas_addr.prv[7];
+  client->addr.prefix[8] = ouraddr.sas_addr.prv[8];
+  client->addr.prefix[9] = ouraddr.sas_addr.prv[9];
+  client->addr.prefix[10] = ouraddr.sas_addr.prv[10];
+  client->addr.prefix[11] = ouraddr.sas_addr.prv[11];
+  client->addr.prefix[12] = ouraddr.sas_addr.prv[12];
+  client->addr.esi[0] = ouraddr.sas_addr.prv[13];
+  client->addr.esi[1] = ouraddr.sas_addr.prv[14];
+  client->addr.esi[2] = ouraddr.sas_addr.prv[15];
+  client->addr.esi[3] = ouraddr.sas_addr.prv[16];
+  client->addr.esi[4] = ouraddr.sas_addr.prv[17];
+  client->addr.esi[5] = ouraddr.sas_addr.prv[18];
+  client->addr.sel = ouraddr.sas_addr.prv[19];
+  client->callback(client->callback_handle,&client->addr, 
+		   ADDR_REG_EVENT_ATM_ADDR_ALLOCATED);
+}
+
 int 
 addr_getesi(unsigned char *mac_addr)
 {
@@ -238,6 +295,9 @@ addr_getesi(unsigned char *mac_addr)
 /*
  *
  * $Log: address.c,v $
+ * Revision 1.12  1996/07/07 11:51:47  carnil
+ * Address changes are noticed and notified
+ *
  * Revision 1.11  1996/04/25 19:42:13  carnil
  * Copyright notice
  *
