@@ -25,7 +25,7 @@
  *
  * Connection forming/teardown & packet send/receive
  *
- * $Id: conn.c,v 1.22 1996/05/29 08:22:39 carnil Exp carnil $
+ * $Id: conn.c,v 1.25 1996/07/07 11:51:47 carnil Exp carnil $
  *
  */
 
@@ -40,6 +40,7 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <assert.h>
+#include <netinet/in.h>
 
 /* Atm includes */
 
@@ -50,6 +51,7 @@
 #include "codes.h"
 #include "g_types.h"
 #include "cm_sap.h"
+#include "lec.h"
 #include "map.h"
 #include "map_data.h"
 #include "atm.h"
@@ -60,10 +62,11 @@
 #include "af_lane.h"
 #include "le_disp.h"
 #include "kernel_itf.h"
+#include "lec_ctrl.h"
 
 #define EMOD MOD_CM
 #define EINST "conn.c"
-#define EMASK (EM_SERR | EM_NERR | EM_WARN | EM_ASSERT | EM_TRAN | EM_EVENT | EM_TRACE | EM_TIMER | EM_MEM | EM_MSG | EM_XCTRL | EM_RCTRL)
+#include "emask.h"
 
 #define BUFSIZE 4096      /* Buffer size. LANE Control packets aren't
 			     supposed to be big... */
@@ -262,6 +265,13 @@ conn_already_exists(unsigned char *atm_addr, Conn_t *current)
   return NULL;
 }
 
+static int
+is_data_direct(CONN_INFO *p_conn_info)
+{
+  return (p_conn_info->blli.l3.tr9577.snap[4] == 0x02 ||
+	  p_conn_info->blli.l3.tr9577.snap[4] == 0x03);
+}
+
 STATUS 
 cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 		  CONN_INFO *p_conn_info, UINT32 age_limit,
@@ -305,11 +315,20 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   s = socket(PF_ATMSVC, SOCK_DGRAM, ATM_AAL5);
   if (s<0) {
     EVENT(EM_NERR,("socket creation failure:%s\n",strerror(errno)));
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+    }
     return STATUS_K_RESOURCES;
   }
   /* Initialize address structs */
   memset(&us,0,sizeof(us));
   if (get_ouratmaddr(s,&us) <0) {
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+    }
+
     close(s);
     return STATUS_K_ATM_RESOURCES;
   }
@@ -318,11 +337,18 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   us.sas_addr.blli = &blli;
   /* Copy blli information */
   memcpy(&blli, &(p_conn_info->blli), sizeof(struct atm_blli));
-  /* Copy traffic parameters */
-  memcpy(&(us.sas_txtp), &(p_conn_info->addr.sas_txtp), 
-			      sizeof(struct atm_trafprm));
-  memcpy(&(us.sas_rxtp), &(p_conn_info->addr.sas_rxtp), 
-			      sizeof(struct atm_trafprm));
+  /* Set traffic parameters */
+  if (setsockopt(s,SOL_ATM,SO_ATMQOS, &(p_conn_info->conqos), 
+		 sizeof(p_conn_info->conqos)) < 0) {
+    EVENT(EM_NERR,("setsockopt SO_ATMQOS: %s\n",strerror(errno)));
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+    }
+    close(s);
+    return STATUS_K_ATM_RESOURCES;
+  }
+  
   us.sas_family = p_conn_info->addr.sas_family;
 
   if (EMASK & EM_DEBUG)
@@ -331,6 +357,10 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   ret = bind(s, (struct sockaddr *)&us, sizeof(us));
   if (ret<0) {
     EVENT(EM_NERR,("bind error:%s\n",strerror(errno)));
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+    }
+
     close(s);
     return STATUS_K_ATM_RESOURCES;
   }
@@ -365,6 +395,10 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 		sizeof(struct sockaddr_atmsvc));
   if (ret<0) {
     EVENT(EM_NERR,("connect error:%s\n",strerror(errno)));
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+    }
     close(s);
     return STATUS_K_ATM_RESOURCES;
   }
@@ -379,6 +413,10 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
     if (ioctl(s, ATMLEC_DATA, &ioc_data)<0) {
       EVENT(EM_SERR,("Can't change socket into LE data socket:%s\n",
 		     strerror(errno)));
+      if (is_data_direct(p_conn_info)) {
+	/* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+      }
       close(s);
       return STATUS_K_ATM_RESOURCES;
     }
@@ -388,6 +426,10 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
     if (ioctl(s, ATMLEC_MCAST)<0) {
       EVENT(EM_SERR,("Can't change socket into LE mcast socket:%s\n",
 		     strerror(errno)));
+      if (is_data_direct(p_conn_info)) {
+	/* Try to remove possible entry in kernel */
+	lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+      }
       close(s);
       return STATUS_K_ATM_RESOURCES;
     }
@@ -401,6 +443,10 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
 		      age_limit, p_conn_info, &conn);
   EVENT(EM_DEBUG,("Conn:%lx\tcontext:%lx\n",(long)conn, (long)conn_context));
   if (ret <0) {
+    if (is_data_direct(p_conn_info)) {
+      /* Try to remove possible entry in kernel */
+      lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
+    }
     close(s);
     return STATUS_K_RESOURCES;
   }
@@ -462,16 +508,20 @@ cm_sap_xmt_vc (HANDLE conn_handle, void *p_data,
   return STATUS_K_SUCCESS;
 }
 
-/* Change!!! */
 void
 cm_sap_link_status_get(HANDLE sap_handle, ATM_LINK_STATE *p_link_state)
 {
-  EVENT(EM_ASSERT,("Cm_sap_link_status_get\n"));
-  assert(sap_handle);
-  /* Is there an interface? */
-  /* Is signalling daemon up? */
-  /* Can we get our atm address? */
+  int s;
 
+  EVENT(EM_DEBUG,("Cm_sap_link_status_get\n"));
+  assert(sap_handle);
+
+  if ((s=socket(AF_ATMSVC, SOCK_DGRAM, ATM_AAL5))<0) {
+    /* Socket creation failure -> not capable of running LE */
+    EVENT(EM_SERR,("Not capable of creating ATM socket."));
+    *p_link_state = LINK_DOWN;
+    exit(-1);
+  }
   *p_link_state = LINK_SIG_UP;
 }
 
@@ -538,7 +588,7 @@ conn_call_callbacks(void)
   }
 }
 
-static char*
+static const char*
 conn_get_type_string(int type) {
   switch(type) {
   case WEMADE:
@@ -683,7 +733,7 @@ conn_check_incoming(fd_set *fds)
 		if (tmpconn) {
 		  get_ouratmaddr(conn->fd, &addr);
 		  if (memcmp(addr.sas_addr.prv, ioc_data.atm_addr, ATM_ESA_LEN)
-		      !=0)
+		      <0)
 		    ioc_data.receive = 1;
 		  else
 		    ioc_data.receive = 0;
@@ -704,7 +754,12 @@ conn_check_incoming(fd_set *fds)
 	if (nbytes <0) {
 	  EVENT(EM_SERR,("Read failed on fd %d: %s\n",
 			 conn->fd,strerror(errno)));
-	  return -1;
+	  if (errno == EUNATCH || errno == ENOENT) 
+	    return -1;
+	  else {
+	    conn->status = RELEASED;
+	    EVENT(EM_EVENT, ("Closing connection\n"));
+	  }
 	} else if (nbytes==0) {
 	  /* EOF, connection closed */
 	  conn->status = RELEASED;
@@ -750,13 +805,37 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
  
   memset(conn_info , 0, sizeof(CONN_INFO));
 
+  conn_info->conqos.txtp.class = ATM_UBR;
+  conn_info->conqos.rxtp.class = ATM_UBR;
+  conn_info->conqos.txtp.max_pcr = max_cell_rate;  
+  conn_info->conqos.rxtp.max_pcr = max_cell_rate;
+
+  switch (blli_codepoint) {
+  case BLLI_CONTROL:
+  case BLLI_DIRECT_802_3:
+  case BLLI_BUS_802_3:
+    conn_info->conqos.rxtp.max_sdu = 1516;
+    conn_info->conqos.txtp.max_sdu = 1516;
+    break;
+  case BLLI_DIRECT_802_5:
+  case BLLI_BUS_802_5:
+    conn_info->conqos.rxtp.max_sdu = 4544;
+    conn_info->conqos.txtp.max_sdu = 4544;
+    break;
+  default:
+    EVENT(EM_ASSERT,("Unknown BLLI codepoint:%x\n",blli_codepoint));
+  }
+
+  /* Set traffic parameters */
+  if (setsockopt(fd,SOL_ATM,SO_ATMQOS, &(conn_info->conqos), 
+		 sizeof(conn_info->conqos)) < 0) {
+    EVENT(EM_NERR,("setsockopt SO_ATMQOS: %s\n",strerror(errno)));
+    close(fd);
+    return -1;
+  }
+  
   conn_info->addr.sas_family = AF_ATMSVC;
   conn_info->addr.sas_addr.blli = &conn_info->blli;
-  conn_info->addr.sas_txtp.class = ATM_UBR;
-  conn_info->addr.sas_rxtp.class = ATM_UBR;
-  conn_info->addr.sas_txtp.max_pcr = max_cell_rate;  
-  conn_info->addr.sas_rxtp.max_pcr = max_cell_rate;
-
   conn_info->blli.l3_proto = ATM_L3_TR9577;
   conn_info->blli.l3.tr9577.ipi = NLPID_IEEE802_1_SNAP;
   conn_info->blli.l3.tr9577.snap[0] = 0x00;
@@ -766,22 +845,6 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
     (unsigned char)(0xff&(blli_codepoint>>8));
   conn_info->blli.l3.tr9577.snap[4] = (unsigned char)(0xff&blli_codepoint);
     
-  switch (blli_codepoint) {
-  case BLLI_CONTROL:
-  case BLLI_DIRECT_802_3:
-  case BLLI_BUS_802_3:
-    conn_info->addr.sas_rxtp.max_sdu = 1516;
-    conn_info->addr.sas_txtp.max_sdu = 1516;
-    break;
-  case BLLI_DIRECT_802_5:
-  case BLLI_BUS_802_5:
-    conn_info->addr.sas_rxtp.max_sdu = 4544;
-    conn_info->addr.sas_txtp.max_sdu = 4544;
-    break;
-  default:
-    EVENT(EM_ASSERT,("Unknown BLLI codepoint:%x\n",blli_codepoint));
-  }
-
   len = sizeof(struct sockaddr_atmsvc);  
   ret = bind(fd, (struct sockaddr *)&conn_info->addr, len);
   if (ret != 0) {
@@ -814,6 +877,19 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
   return fd;
 }
 
+void 
+cm_reset(HANDLE sap_handle)
+{
+  Conn_t *conn, *next;
+
+  /* Remove vcc's */
+  for(conn=connlist;conn!=NULL;conn=next) {
+    next = conn->next;
+    close(conn->fd);
+    list_remove_conn(conn);
+  }
+}
+
 static int
 list_add_conn(HANDLE sap_handle, HANDLE conn_context, long age_limit, 
 	      CONN_INFO *info, Conn_t **p_conn)
@@ -879,19 +955,7 @@ conn_set_kernel_socket(int fd)
   conn->fd = fd;
   return fd;
 }
-/*
-void 
-conn_convert_to_data_vcc(HANDLE conn_handle)
-{
-  Conn_t *conn;
 
-  conn = (Conn_t *)conn_handle;
-
-  if (ioctl(conn->fd, ATMLEC_DATA)<0) {
-    EVENT(EM_NERR,("Failed to convert vcc into LEC data\n"));
-  }
-}
-*/
 int
 get_ouratmaddr(int s, struct sockaddr_atmsvc *addr)
 {
@@ -911,6 +975,14 @@ get_ouratmaddr(int s, struct sockaddr_atmsvc *addr)
 /*
  *
  * $Log: conn.c,v $
+ * Revision 1.25  1996/07/07 11:51:47  carnil
+ * Global msg mask
+ *
+ * Revision 1.24  1996/06/10 04:27:28  carnil
+ * traffic parameter via setsockopt
+ * compiler warning fix
+ * Bug fixes
+ *
  * Revision 1.22  1996/05/29 08:22:39  carnil
  * bug fix
  *
