@@ -468,46 +468,6 @@ void lc_join_start (HANDLE  lc_elan_handle,
                     HANDLE  mcast_conn_context,
                     HANDLE  ctrl_conn_context);
 
-
-/*++
-* ===================
-* = lc_addr_resolve =
-* ===================
-*
-* Overview:
-*   Requests the resolution of a MAC address to a connection.  This lookup is
-*   used to determine which connection a packet is to be transmitted on.  If
-*   the destination address is multicast, the resolution is to the multicast-
-*   send VCC.
-*
-* Arguments:
-*   lc_elan_handle  - (IN)  Handle returned by lc_register.
-*
-*   dst_addr        - (IN)  MAC destination address to be looked up.
-*
-*   p_may_xmt       - (OUT) Flag indicating that the packet may be transmitted.
-*                           If this flag is TRUE, the packet may be
-*                           transmitted.  If it is FALSE, the packet must not
-*                           be transmitted for reasons of "congestion".  This
-*                           is normally due to flooding rate limits.
-*
-*   p_conn_handle   - (OUT) Connection Handle to be used for the transmit.
-*
-* Returns:
-*   None
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-void lc_addr_resolve (HANDLE   lc_elan_handle,
-                      ESI      dst_addr,
-                      BOOLEAN *p_may_xmt,
-                      HANDLE  *p_conn_handle);
-
-
 /*++
 * ====================
 * = lc_sap_vc_notify =
@@ -580,81 +540,14 @@ void lc_sap_rcv (HANDLE     conn_context,
 void lc_sap_link_status (HANDLE          lc_elan_handle,
                          ATM_LINK_STATE  link_state);
 
-/*++
-* ===============
-* = lc_attr_get =
-* ===============
-*
-* Overview:
-*   Get a management attribute for an ELAN instance.  Refer to lec_mgt.h for
-*   a list of attribute IDs.
-*
-* Arguments:
-*   lc_elan_handle    - (IN)  Handle returned by lc_register.
-*   attr_id           - (IN)  ID of attribute to be accessed.
-*   p_out_buffer      - (IN)  Pointer to the output buffer to be filled with
-*                             data.
-*   out_buffer_length - (IN)  Number of bytes writable in the output buffer.
-*   p_bytes_written   - (OUT) Number of bytes written.
-*
-* Returns:
-*   STATUS_K_SUCCESS       - Successful Completion.
-*   STATUS_K_ARG_2_BAD     - Invalid Attribute ID.
-*   STATUS_K_ARG_4_BAD     - Not enough space in output buffer.
-*   STATUS_K_NOT_FOUND     - Instance not found.
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-STATUS lc_attr_get (HANDLE     lc_elan_handle,
-                    UINT16     attr_id,
-                    void      *p_out_buffer,
-                    UINT16     out_buffer_length,
-                    UINT16    *p_bytes_written);
 
-STATUS lc_attr_get_next (HANDLE     lc_elan_handle,
-                         UINT16     attr_id,
-                         void      *p_in_buffer,
-                         UINT16     in_buffer_length,
-                         void      *p_out_buffer,
-                         UINT16     out_buffer_length,
-                         UINT16    *p_bytes_written);
-
-/*++
-* ===============
-* = lc_attr_set =
-* ===============
-*
-* Overview:
-*   Set a management attribute for an ELAN instance.  Refer to lec_mgt.h for
-*   a list of attribute IDs.
-*
-* Arguments:
-*   lc_elan_handle    - (IN)  Handle returned by lc_register.
-*   attr_id           - (IN)  ID of attribute to be accessed.
-*   p_in_buffer       - (IN)  Pointer to the input buffer that contains data
-*                             to be written.
-*   in_buffer_length  - (IN)  Number of bytes to be written.
-*
-* Returns:
-*   STATUS_K_SUCCESS       - Successful Completion.
-*   STATUS_K_ARG_2_BAD     - Invalid Attribute ID.
-*   STATUS_K_ARG_5_BAD     - Not enough space in output buffer.
-*   STATUS_K_NOT_FOUND     - Instance not found.
-*   STATUS_K_NO_PRIVELEGE  - Attempted to write a Read-Only attribute.
-*
-* Preconditions:
-*   None
-*
-* Postconditions:
-*   None
---*/
-STATUS lc_attr_set (HANDLE     lc_elan_handle,
-                    UINT16     attr_id,
-                    void      *p_in_buffer,
-                    UINT16     in_buffer_length);
+/* 
+ * Forwards address deletion request to lec_arp module.
+ */
+void lc_addr_delete(HANDLE lc_elan_handle, unsigned char *atm_addr);
 
 #endif /* LEC_CTRL_H */
+
+
+
+
