@@ -1,6 +1,6 @@
 /* atmdump.c - ATM raw cell dumper */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -56,6 +56,7 @@ int main(int argc,char **argv)
 {
     unsigned long buf[13]; /* ugly */
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     char *name,*end;
     int type,gfc,clp;
     int c,s,size;
@@ -87,13 +88,18 @@ int main(int argc,char **argv)
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[optind],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC)
       < 0) usage(name);
+    memset(&qos,0,sizeof(qos));
     if (type == -1) {
-	addr.sap_rxtp.class = ATM_UBR;
-	addr.sap_rxtp.max_sdu = 52;
+	qos.rxtp.class = ATM_UBR;
+	qos.rxtp.max_sdu = 52;
     }
     else {
-	addr.sap_txtp.class = ATM_UBR;
-	addr.sap_txtp.max_sdu = 52;
+	qos.txtp.class = ATM_UBR;
+	qos.txtp.max_sdu = 52;
+    }
+    if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
+	perror("setsockopt SO_ATMQOS");
+	return 1;
     }
     if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
