@@ -1,6 +1,6 @@
 /* op.h - message processor opcodes */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #ifndef OP_H
@@ -28,4 +28,6 @@
 			     <group>,<offset>,... { <op>,...,jump }, ... } */
 #define OP_IFEND       10 /* ifend <addr> */
 
+#define OP_DUMP	       11 /* dump <index> */
+
 #endif
