@@ -8,15 +8,28 @@ int q_dump = 0;
 
 #ifdef DUMP_MODE
 
+#include <stdarg.h>
 #include <stdio.h>
+#include <unistd.h>
 
 #include "common.h"
 #include "qlib.h"
 #include "op.h"
 
 
+static int debug = 0;
+
+
 void q_report(int severity,const char *msg,...)
 {
+    va_list ap;
+
+    if (!debug) return;
+    va_start(ap,msg);
+    vprintf(msg,ap);
+    printf("\n");
+    va_end(ap);
+    if (severity == Q_FATAL) exit(1);
 }
 
 #endif
@@ -45,7 +58,14 @@ static void q_put(unsigned char *table,int pos,int size,unsigned long value)
     end = pos+size;
     if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
 	q_report(Q_FATAL,"unsupported alignment");
-    if (size <= 8) table[pos >> 3] |= value << (pos & 7);
+    if (size <= 8) {
+	unsigned char *here;
+	int shift;
+
+	here = &table[pos >> 3];
+	shift = pos & 7;
+	*here = (*here & ~(((1 << size)-1) << shift)) | value << shift;
+    }
     else {
 	table = table+end/8-1;
 	while (size > 0) {
@@ -64,10 +84,8 @@ static unsigned long q_get(unsigned char *table,int pos,int size)
  
     q_report(Q_DEBUG,"get %d %d ...",pos,size);
     end = pos+size;
-    if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7)) {
+    if (((pos | size) & 7) && ((pos ^ (end-1)) & ~7))
 	q_report(Q_FATAL,"unsupported alignment");
-	return 0;
-    }
     if (size <= 8) value = (table[pos >> 3] >> (pos & 7)) & ((1 << size)-1);
     else {
 	table += pos >> 3;
@@ -195,10 +213,8 @@ void q_assign(Q_DSC *dsc,int field,unsigned long value)
 	use_group(dsc,fields[field].parent);
     }
     else {
-	if (q_test(dsc->field_present,field)) {
+	if (q_test(dsc->field_present,field))
 	    q_report(Q_FATAL,"can't change field %d",field);
-	    return;
-	}
 	q_set(dsc->field_present,field);
 	q_put(dsc->data,fields[field].pos,fields[field].size,value);
 	for (walk = fields[field].values; walk[1] != -1; walk += 2)
@@ -206,41 +222,37 @@ void q_assign(Q_DSC *dsc,int field,unsigned long value)
 		use_group(dsc,walk[1]);
 		return;
 	    }
-	q_report(Q_FATAL,"invalid value (%d in field %d)",value,field);
+	q_report(Q_ERROR,"invalid value (%d in field %d)",value,field);
+	dsc->error = 1;
     }
 }
 
 
-void q_write(Q_DSC *dsc,int field,void *buf,int size)
+void q_write(Q_DSC *dsc,int field,const void *buf,int size)
 {
     if (field < 0 || field >= Q_FIELDS)
 	q_report(Q_FATAL,"invalid field value (%d)",field);
-    if (fields[field].pos & 7) {
+    if (fields[field].pos & 7)
 	q_report(Q_FATAL,"invalid use of q_write (%d)",field);
-	return;
-    }
     if (fields[field].actual >= 0) {
 	if (size > fields[field].size/8) {
-	    q_report(Q_FATAL,"%d bytes too big for %d byte field %d",size,
+	    q_report(Q_ERROR,"%d bytes too big for %d byte field %d",size,
 	      fields[field].size/8,field);
+	    dsc->error = 1;
 	    return;
 	}
 	dsc->length[fields[field].actual] = size;
     }
-    else {
-	if ((fields[field].pos | fields[field].size) & 7) {
+    else if ((fields[field].pos | fields[field].size) & 7)
 	    q_report(Q_FATAL,"field %d is neither var-len nor well-shaped",
 	      field);
-	    return;
-	}
-    }
     memcpy(dsc->data+(fields[field].pos/8),buf,(size_t) size);
     q_set(dsc->field_present,field);
     use_group(dsc,fields[field].parent);
 }
 
 
-int q_present(Q_DSC *dsc,int field)
+int q_present(const Q_DSC *dsc,int field)
 {
     if (field < 0) {
 	if (field < -Q_GROUPS)
@@ -256,7 +268,7 @@ int q_present(Q_DSC *dsc,int field)
 }
 
 
-unsigned long q_fetch(Q_DSC *dsc,int field)
+unsigned long q_fetch(const Q_DSC *dsc,int field)
 {
     if (field < 0 || field >= Q_FIELDS)
 	q_report(Q_FATAL,"invalid field value (%d)",field);
@@ -264,42 +276,38 @@ unsigned long q_fetch(Q_DSC *dsc,int field)
 }
 
 
-int q_length(Q_DSC *dsc,int field)
+int q_length(const Q_DSC *dsc,int field)
 {
     if (field < 0 || field >= Q_FIELDS)
 	q_report(Q_FATAL,"invalid field value (%d)",field);
-    if (fields[field].pos & 7) {
+    if (fields[field].pos & 7)
 	q_report(Q_FATAL,"invalid use of q_length (%d)",field);
-	return -1;
-    }
-    if (fields[field].actual < 0) {
+    if (fields[field].actual < 0)
 	q_report(Q_FATAL,"field %d is not var-len",field);
-	return -1;
-    }
     return dsc->length[fields[field].actual];
 }
 
 
 int q_read(Q_DSC *dsc,int field,void *buf,int size)
 {
+    int len;
+
     if (field < 0 || field >= Q_FIELDS)
 	q_report(Q_FATAL,"invalid field value (%d)",field);
-    if (fields[field].pos & 7) {
+    if (fields[field].pos & 7)
 	q_report(Q_FATAL,"invalid use of q_read (%d)",field);
+    if (fields[field].actual >= 0) len = dsc->length[fields[field].actual];
+    else if (!(fields[field].size & 7)) len = fields[field].size >> 3;
+	else q_report(Q_FATAL,"field %d is not byte-sized (%d bits)",field,
+	      fields[field].size);
+    if (size < len) {
+	q_report(Q_ERROR,"%d bytes too big for %d byte buffer (field %d)",len,
+	  size,field);
+	dsc->error = 1;
 	return -1;
     }
-    if (fields[field].actual < 0) {
-	q_report(Q_FATAL,"field %d is not var-len",field);
-	return -1;
-    }
-    if (size < dsc->length[fields[field].actual]) {
-	q_report(Q_FATAL,"%d bytes too big for %d byte buffer (field %d)",
-	  dsc->length[fields[field].actual],size,field);
-	return -1;
-    }
-    memcpy(buf,dsc->data+(fields[field].pos/8),
-      dsc->length[fields[field].actual]);
-    return dsc->length[fields[field].actual];
+    memcpy(buf,dsc->data+(fields[field].pos/8),len);
+    return len;
 }
 
 
@@ -337,8 +345,9 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 	switch (*pc++) {
 	    case OP_COPY:
 		if (size < *pc) {
-		    q_report(Q_FATAL,"not enough space (%d < %d)",size,*pc);
-		    return 01;
+		    q_report(Q_ERROR,"not enough space (%d < %d)",size,*pc);
+		    dsc->error = 1;
+		    return -1;
 		}
 		q_report(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
 		q_copy(dsc->data,pc[1],pos,pc[2]);
@@ -352,8 +361,9 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		break;
 	    case OP_COPYVAR:
 		if (size < dsc->length[*pc]) {
-		    q_report(Q_FATAL,"not enough space (%d < %d)",size,
+		    q_report(Q_ERROR,"not enough space (%d < %d)",size,
 		      dsc->length[*pc]);
+		    dsc->error = 1;
 		    return -1;
 		}
 		memcpy(pos,dsc->data+pc[1]/8,dsc->length[*pc]);
@@ -363,11 +373,13 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		break;
 	    case OP_BEGIN_LEN:
 		if (size < *pc) {
-		    q_report(Q_FATAL,"not enough space (%d < %d)",size,*pc);
+		    q_report(Q_ERROR,"not enough space (%d < %d)",size,*pc);
+		    dsc->error = 1;
 		    return -1;
 		}
 		if (sp == LENGTH_STACK) {
-		    q_report(Q_FATAL,"length stack overflow");
+		    q_report(Q_ERROR,"length stack overflow");
+		    dsc->error = 1;
 		    return -1;
 		}
 		stack[sp].pos = pc[1]; /* not used */
@@ -379,10 +391,7 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		pc += 3;
 		break;
 	    case OP_END_LEN:
-		if (!sp--) {
-		    q_report(Q_FATAL,"length stack underflow");
-		    return -1;
-		}
+		if (!sp--) q_report(Q_FATAL,"length stack underflow");
 		q_put(stack[sp].start,0,stack[sp].size,
 		  (size_t) ((pos-stack[sp].start)-((stack[sp].size+7) >> 3)));
 		break;
@@ -401,10 +410,8 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 			    pc += *pc+1;
 			    break;
 			}
-		    if (!len) {
+		    if (!len)
 			q_report(Q_FATAL,"multi failed (pc %d)",pc-construct);
-			return -1;
-		    }
 		}
 		break;
 #endif
@@ -415,7 +422,7 @@ static int q_compose(Q_DSC *dsc,unsigned char *buf,int size)
 		return pos-buf;
 	    default:
 		q_report(Q_FATAL,"unrecognized opcode %d",pc[-1]);
-		return -1;
+		return -1; /* for gcc */
 	}
     }
 }
@@ -468,8 +475,9 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 #endif
 	    case OP_COPY:
 		if (pos+*pc > end) {
-		    q_report(Q_FATAL,"not enough space (%d+%d > %d)",pos-buf,
+		    q_report(Q_ERROR,"not enough space (%d+%d > %d)",pos-buf,
 		      *pc,end-buf);
+		    dsc->error = 1;
 		    return -1;
 		}
 		q_report(Q_DEBUG,"copy %d %d %d",pc[1],pos-buf,pc[2]);
@@ -497,29 +505,25 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		}
 	    case OP_BEGIN_LEN:
 		if (pos+*pc > end) {
-		    q_report(Q_FATAL,"not enough space (%d+%d > %d)",pos-buf,
+		    q_report(Q_ERROR,"not enough space (%d+%d > %d)",pos-buf,
 		      *pc,end-buf);
+		    dsc->error = 1;
 		    return -1;
 		}
 		if (sp == LENGTH_STACK) {
-		    q_report(Q_FATAL,"length stack overflow");
+		    q_report(Q_ERROR,"length stack overflow");
+		    dsc->error = 1;
 		    return -1;
 		}
 		stack[sp] = end;
 		end = pos+q_get(pos,pc[1] & 7,pc[2])+*pc;
-		if (end > stack[sp]) {
-		    q_report(Q_FATAL,"length has grown");
-		    return -1;
-		}
+		if (end > stack[sp]) q_report(Q_FATAL,"length has grown");
 		pos += *pc;
 		sp++;
 		pc += 3;
 		break;
 	    case OP_END_LEN:
-		if (!sp--) {
-		    q_report(Q_FATAL,"length stack underflow");
-		    return -1;
-		}
+		if (!sp--) q_report(Q_FATAL,"length stack underflow");
 		end = stack[sp];
 		break;
 	    case OP_CASE:
@@ -527,8 +531,9 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		    int len,value,group;
 
 		    if (pos+*pc > end) {
-			q_report(Q_FATAL,"not enough space (%d+%d > %d)",
+			q_report(Q_ERROR,"not enough space (%d+%d > %d)",
 			  pos-buf,*pc,end-buf);
+			dsc->error = 1;
 			return -1;
 		    }
 		    value = q_get(pos,pc[1] & 7,pc[2]);
@@ -562,7 +567,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 		return 0;
 	    default:
 		q_report(Q_FATAL,"unrecognized opcode %d",pc[-1]);
-		return -1;
+		return -1; /* for gcc */
 	}
     }
 }
@@ -570,6 +575,7 @@ static int _q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 
 static int q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 {
+    int error;
     int j,i;
     int *p;
 
@@ -579,7 +585,8 @@ static int q_parse(Q_DSC *dsc,unsigned char *buf,int size)
 	    for (i = 0; i < 20; i++) fprintf(stderr," %02X",dsc->data[i+j]);
 	    putc('\n',stderr);
 	}
-    _q_parse(dsc,buf,size);
+    error = _q_parse(dsc,buf,size);
+    if (error) return error;
     if (q_dump) {
 	putc('\n',stderr);
 	for (j = 0; j < 100; j += 20) {
@@ -602,8 +609,7 @@ int q_open(Q_DSC *dsc,void *buf,int size)
 {
     dsc->buffer = NULL;
     q_init(dsc);
-    q_parse(dsc,buf,size);
-    return 0;
+    return q_parse(dsc,buf,size);
 }
 
 
@@ -620,27 +626,30 @@ int q_close(Q_DSC *dsc)
 {
     int size;
 
-    if (dsc->buffer) size = q_compose(dsc,dsc->buffer,dsc->buf_size);
+    if (dsc->buffer && !dsc->error)
+	size = q_compose(dsc,dsc->buffer,dsc->buf_size);
     free(dsc->data);
     free(dsc->required);
     free(dsc->field_present);
     free(dsc->group_present);
     if (dsc->length) free(dsc->length);
-    return dsc->buffer ? size : 0;
+    return dsc->error ? -1 : dsc->buffer ? size : 0;
 }
 
 
 #ifdef DUMP_MODE
 
-void main(void)
+int main(int argc,const char **argv)
 {
     unsigned char msg[5000]; /* should be large enough for that */
     Q_DSC dsc;
     int len,c;
 
+    debug = argc != 1;
     len = 0;
     while (scanf("%x",&c) == 1) msg[len++] = c;
     q_open(&dsc,msg,len);
+    return 0;
 }
 
 #endif
