@@ -132,9 +132,12 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	case as_bind: /* only in NULL state */
 	    if (sock) break;
 	    if (!*msg->svc.sas_addr.pub && !*msg->svc.sas_addr.prv)
+#ifdef BE_PICKY_ABOUT_BINDING_LOCAL_WILDCARD_ADDRESSES
 		if (local_addr[0].state != ls_same)
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
-		else send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,
+		else
+#endif
+		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,
 		      &local_addr[0].addr);
 	    else {
 		LOCAL_ADDR *walk;
