@@ -41,6 +41,11 @@ const Q2931_STATE state_map[] = { /* formatting aligned with STATE */
 	qs_active,	qs_rel_req,	qs_null,	qs_rel_req,
 	qs_rel_ind };
 
+const PARTY_STATE eps_map[] = {
+	ps_null,	ps_add_init,	ps_null,	ps_add_init,	/* 0 */
+	ps_null,	ps_null,	ps_add_recv,	ps_null,	/* 4 */
+	ps_active,	ps_add_recv,	ps_active,	ps_active,	/* 8 */
+	ps_active };							/*12 */
 
 SOCKET *sockets = NULL;
 unsigned char q_buffer[MAX_Q_MSG];
@@ -58,6 +63,7 @@ SOCKET *new_sock(unsigned long id)
     sock->local = sock->remote = NULL;
     sock->error = 0;
     sock->q2931_state = qs_null;
+    sock->ep_ref = -1;
     sock->conn_timer = NULL;
     sock->listen = NULL;
     sock->next = sockets;
