@@ -60,7 +60,7 @@ static void setup_call(unsigned long call_ref)
     SOCKET *sock,*this,**walk;
     struct sockaddr_atmsvc *sap,*new_sap;
     unsigned char buf[ATM_ESA_LEN];
-    int len,i,vpci;
+    int len,i;
 
     sap = sap_decode(&in_dsc);
     new_sap = NULL;
@@ -87,10 +87,14 @@ static void setup_call(unsigned long call_ref)
 	for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
 	putchar('\n');
     }
-    vpci = q_fetch(&in_dsc,QF_vpi);
-    this->pvc.sap_addr.itf = get_itf(&vpci);
-    this->pvc.sap_addr.vpi = vpci;
-    this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
+    if (!net) { /* already set by send_call_proceeding */
+	int vpci;
+
+	vpci = q_fetch(&in_dsc,QF_vpi);
+	this->pvc.sap_addr.itf = get_itf(&vpci);
+	this->pvc.sap_addr.vpi = vpci;
+	this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
+    }
     diag(COMPONENT,DIAG_DEBUG,"ITF.VPI.VCI: %d.%d.%d",this->pvc.sap_addr.itf,
       this->pvc.sap_addr.vpi,this->pvc.sap_addr.vci);
     this->remote = alloc_t(struct sockaddr_atmsvc);
@@ -229,6 +233,12 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    {
 		Q2931_STATE state;
 
+		/*
+		 * NOTE: T322 isn't implemented yet, but when it is, make sure
+		 *	 to only stop it on STATUS iff the cause is
+		 *	 ATM_CV_RESP_STAT_ENQ. Supplementary services break if
+		 *	 you stop on any STATUS.
+		 */
 		if (sock->q2931_state == qs_rel_req || sock->q2931_state ==
 		  qs_rel_ind) return;
 		state = q_fetch(&in_dsc,QF_call_state);
