@@ -86,7 +86,7 @@ static int bhli_compat(const struct atm_bhli *old,const struct atm_bhli *new,
     int length;
 
     if (res) *res = *new;
-    if (!old->hl_type) return 1;
+/*   if (!old->hl_type) return 1; --- GONE */
     if (old->hl_type != new->hl_type) return 0;
     switch (old->hl_type) {
 	case ATM_HL_ISO:
