@@ -27,6 +27,7 @@ static void usage(const char *name)
 int main(int argc,char **argv)
 {
     struct sockaddr_atmpvc addr;
+    struct atm_qos qos;
     char buffer[BSIZE];
     int blocks,s;
     size_t size;
@@ -41,8 +42,13 @@ int main(int argc,char **argv)
     memset(&addr,0,sizeof(addr));
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
-    addr.sap_txtp.class = ATM_UBR;
-    addr.sap_txtp.max_sdu = BSIZE;
+    memset(&qos,0,sizeof(qos));
+    qos.txtp.class = ATM_UBR;
+    qos.txtp.max_sdu = BSIZE;
+    if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
+	perror("setsockopt SO_ATMQOS");
+	return 1;
+    }
     if (connect(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
 	perror("bind");
 	return 1;
